<?php

/**/
function deepl_get_languages($auth_key, &$language_array = null) {
	$url = 'https://api.deepl.com/v2/languages?auth_key='.$auth_key;
	$data = array('auth_key' => $auth_key, 'type' => 'target');
	$data = http_build_query($data);

	$options = array(
		'http' => array(
			'method'  => 'POST',
			'header'  => array('Content-type: application/x-www-form-urlencoded',
								'User-Agent: GetSimple DeepL Translator Plugin V.1.0',
								'Accept: *'.'/'.'*',
								'Content-Length: ' . strlen($data)					
						),
			'content' => $data
		)
	);
	$context  = stream_context_create($options);
	$lang_array = json_decode(file_get_contents($url, false, $context),true);
	$statuscode = intval(preg_replace('{HTTP\/\S*\s(\d{3}).*}', '${1}', $http_response_header[0]));

	$language_array = [];
	foreach($lang_array as $lang) {
		$language = [];
		$language['cmd'] = $lang['language'];
		$language['description'] = $lang['name'];
		$language['menu'] = $lang['name'];
		$language['postfix'] = '_'.mb_strtolower($lang['language'], 'UTF-8');
		$language_array[] = $language;
	}
	return $statuscode;
}

function deepl_get_usage($auth_key, &$usage_array = null) {
	$url = 'https://api.deepl.com/v2/usage?auth_key='.$auth_key;
	$data = array('auth_key' => $auth_key);
	$data = http_build_query($data);

	$options = array(
		'http' => array(
			'method'  => 'POST',
			'header'  => array('Content-type: application/x-www-form-urlencoded',
								'User-Agent: GetSimple DeepL Translator Plugin V.1.0',
								'Accept: *'.'/'.'*',
								'Content-Length: ' . strlen($data)					
						),
			'content' => $data
		)
	);
	$context  = stream_context_create($options);
	$usage_arr = json_decode(file_get_contents($url, false, $context),true);
	$statuscode = intval(preg_replace('{HTTP\/\S*\s(\d{3}).*}', '${1}', $http_response_header[0]));
	if ($statuscode == 200) {
		$usage_array = [];
		$usage_array['USAGE_COUNT'] = $usage_arr['character_count'];
		$usage_array['USAGE_LIMIT'] = $usage_arr['character_limit'];
		$usage_array['USAGE_REMAINING'] = $usage_array['USAGE_LIMIT'] - $usage_array['USAGE_COUNT'];
	}
	return $statuscode;
}

/**/

function deepl_error_description($err) {
	$deepl_errors = [
		400 => 'DEEPL_ERR_400',
		403 => 'DEEPL_ERR_403',
		404 => 'DEEPL_ERR_404',
		413 => 'DEEPL_ERR_413',
		414 => 'DEEPL_ERR_414',
		429 => 'DEEPL_ERR_429',
		456 => 'DEEPL_ERR_456',
		503 => 'DEEPL_ERR_503'
	];
	if (array_key_exists($err, $deepl_errors)) {
		return ti18n_r('DEEPL_ERR_'.$err.'');
	} else if ($err >= 500) {
		return ti18n_r('DEEPL_ERR_INTERNAL');
	} else {
		return ti18n_r('DEEPL_ERR_OTHER');
	}
}

function deepl_translate($auth_key, $dest_lang, $settings, $fields_arr, &$translated_array = null) {
	$url = 'https://api.deepl.com/v2/translate?auth_key='.$auth_key;
	$around = '_@around@__';
	$notrans_name = '_@name@__';
	$content = '_@content@__';
	$notrans_other = '_@other@__';
	$settings['deepl']['ignore_tags'] .= ','.$notrans_name.','.$notrans_other;
	$settings['deepl']['splitting_tags'] .= ','.$around.','.$notrans_name;
	$settings['deepl']['non_splitting_tags'] .= ','.$notrans_other;
	$around_start = '<'.$around.'>';
	$around_stop = '</'.$around.'>';
	$name_start = '<'.$notrans_name.'>';
	$name_stop = '</'.$notrans_name.'>';
	$content_start = '<'.$content.'>';
	$content_stop = '</'.$content.'>';
	$other_start = '<'.$notrans_other.'>';
	$other_stop = '</'.$notrans_other.'>';

	$omits = config_explode($settings['config']['omit_translate']);
	$omits_tagged = [];
	foreach ($omits as $omitword) {
		$omits_tagged[$omitword] = $other_start.$omitword.$other_stop;
	}

	$text='';
	foreach ($fields_arr as $fieldname => $fieldtext) {
		$fieldtext = strtr($fieldtext, $omits_tagged);
		if ($settings['config']['omit_placeholders'] === 'Y') {
			$fieldtext = preg_replace('/(\(%.*?%\)|\{%.*?%\}|%[^ \t\r\n%]++%)/u', $other_start.'\1'.$other_stop, $fieldtext);
		}
		$text .= $around_start.$name_start.$fieldname.$name_stop.$content_start.$fieldtext.$content_stop.$around_stop;
	}
	$data = [];
	$data['auth_key']     = $auth_key;
	$data['text']         = $text;
	$data['target_lang']  = $dest_lang;
	foreach($settings['deepl'] as $param => $setting) {
		if ($setting != '') {

			// 2020-09-01: DeepL throws 400 if formality is unsupported!
			if ($param == 'formality') {
				if (in_array($setting, ['EN', 'EN-GB', 'EN-US', 'ES', 'JA', 'ZH'])) continue;
			}

			$data[$param] = $setting;
		}
	}
	$data = http_build_query($data);

	$options = array(
		'http' => array(
			'method'  => 'POST',
			'header'  => array('Content-type: application/x-www-form-urlencoded',
								'User-Agent: GetSimple DeepL Translator Plugin V.1.0',
								'Accept: *'.'/'.'*',
								'Content-Length: ' . strlen($data)					
						),
			'content' => $data
		)
	);
	$context  = stream_context_create($options);
	$response_arr = json_decode(file_get_contents($url, false, $context),true);
	$statuscode = intval(preg_replace('{HTTP\/\S*\s(\d{3}).*}', '${1}', $http_response_header[0]));
	$translated_array = [];
	if ($statuscode == 200) {
		$responsetext = $response_arr['translations'][0]['text'];
		$text_arr = explode($around_start, $responsetext);
		foreach ($text_arr as $textfield) if ( $textfield !== '') {
			$textfield = strtr($textfield, [$around_start => '', $around_stop => '']);
			$t2 = explode($name_stop, $textfield);
			$t2[0] = strtr($t2[0], array($name_start => '', $name_stop => ''));
			$t2[1] = strtr($t2[1], array($content_start => '', $content_stop => ''));

			// 2020-08-29: deepl "eats" necessary spaces directly around "ignore"-tagged text.
			// So in case of an adjacent word, add a space in between. Not always perfect, but probably better.
			$ignoretags = config_explode($settings['deepl']['ignore_tags']);
			foreach ($ignoretags as $ignoretag) {
				$ign_start = '<'.$ignoretag.'>';
				$ign_stop = '</'.$ignoretag.'>';
				$t2[1] = preg_replace('|(\w)'.preg_quote($ign_start,'|').'|u', '${1} '.$ign_start, $t2[1]);
				$t2[1] = preg_replace('|'.preg_quote($ign_stop,'|').'(\w)|u', $ign_stop.' ${1}', $t2[1]);
			}

			$t2[1] = strtr($t2[1], array($other_start => '', $other_stop => ''));
			$translated_array[$t2[0]] = $t2[1];
		}
	} else {
		$translated_array['error'] = ''.$statuscode.': '.deepl_error_description($statuscode);
		$translated_array['message'] = ($responsetext['message'])?: ti18n_r('DEEPL_ERR_NOMESSAGE');
	}
	return $statuscode;
}

/**/


// split a comma-separated settings field into an array
function config_explode( string $input) {

	preg_match_all( '/\s*("[^"]*(?:""[^"]*)*"|[^,]*)\s*,/u', $input.',', $matches ) ;	
	$fields = $matches[1];

	$values = [];
	foreach ($fields as $field) {
		if ( preg_match( '/^"(.*)"$/su', $field, $m ) ) {
			$field = strtr($m[1], array('""' => '"')) ;
		}
		$values[] = $field;
	}
	
	return $values;
}

// combine an array into a comma-separated settings field
function config_implode( array $fields ) {

        $output =[];
        foreach ( $fields as $field ) {

            if (($field === null)||($field === '')) {
               // $output[] = '""';
                continue;
            }
            if (preg_match( '/(?:,|"|\s)/u', $field ?: '' )) {
                $output[] = '"' . strtr($field, array('"' => '""')) . '"';
			}
            else {
                $output[] = $field;
            }
        }
       return implode( ', ', $output );
}

// deletes spaces and newlines from a comma-separated settings fields, for deepl
function deepl_compress( string $input) {

	$input = preg_replace('/\s/u','', $input);
	
	return $input;
}


